@@ -2,6 +2,8 @@ use regex::Regex;
 use thiserror::Error;
 use chrono::Timelike;
 
+use crate::{CrcError, calc_crc};
+
 /// `BIPM` tracking duration specifications.
 /// `Cggtts` tracks must respect that duration
 /// to be BIPM compliant, which is not mandatory 
@@ -89,6 +91,25 @@ pub enum ConstellationRinexCode {
     NonSupportedCode,
 }
 
+impl std::fmt::Display for ConstellationRinexCode {
+    fn fmt (&self, fmt: &mut std::fmt::Formatter) -> std::fmt::Result {
+        let s = match self {
+            ConstellationRinexCode::GPS_GLO_QZ_SBA_L1C => String::from("L1C"),
+            ConstellationRinexCode::GPS_GLO_L1P => String::from("L1P"),
+            ConstellationRinexCode::GAL_E1 => String::from("E1C"),
+            ConstellationRinexCode::GAL_E1E5a => String::from("E5C"),
+            ConstellationRinexCode::QZSS_L1C => String::from("Q1C"),
+            ConstellationRinexCode::BEIDOU_B1i => String::from("B1C"),
+            ConstellationRinexCode::BEIDOU_BliB2i => String::from("B1C"),
+            ConstellationRinexCode::GPS_C1_P1C2_P2 => String::from("P1C"),
+            ConstellationRinexCode::GLO_C1_P1C2_P2 => String::from("Q5C"),
+            ConstellationRinexCode::GZSS_C1C5 => String::from("Q5C"),
+            ConstellationRinexCode::NonSupportedCode => String::from("???"),
+        };
+        fmt.write_str(&s)
+    }
+}
+
 #[derive(Error, Debug)]
 pub enum ConstellationRinexCodeError {
     #[error("unknown constellation code '{0}'")]
@@ -201,31 +222,12 @@ pub enum Error {
     ConstellationRinexCodeError(#[from] ConstellationRinexCodeError),
     #[error("failed to parse common view class")]
     CommonViewClassError(#[from] std::str::Utf8Error),
-    #[error("crc calc() working on non ascii data \"{0}\"")]
-    NonAsciiData(String),
+    #[error("crc calc() failed over non utf8 data: \"{0}\"")]
+    NonAsciiData(#[from] CrcError),
     #[error("checksum error - expecting \"{0}\" - got \"{1}\"")]
     ChecksumError(u8, u8),
 }
 
-
-/// computes crc for given str content
-pub fn calc_crc (content: &str) -> Result<u8, Error> {
-    match content.is_ascii() {
-        true => {
-            let mut ck: u8 = 0;
-            let mut ptr = content.encode_utf16();
-            for _ in 0..ptr.clone().count() {
-                ck = ck.wrapping_add(
-                    ptr.next()
-                    .unwrap()
-                    as u8)
-            }
-            Ok(ck)
-        },
-        false => return Err(Error::NonAsciiData(String::from(content))),
-    }
-}
-
 impl CggttsTrack {
     /// Builds `CggttsTrack` object with
     /// default attributes
@@ -349,13 +351,36 @@ impl std::fmt::Display for CggttsTrack {
         }
         string.push_str(&format!("{: >2}", self.sat_id));
         string.push_str(&format!(" {:02X}", self.class as u8));
+        string.push_str(" 57000");
+
         // self.trktime.format("%H%M%S");
         // let fmt = StrftimeItems::new("%dH%M%S")
         //string.push_str(self.trktime.strptime());
         string.push_str(&format!(" {:0>3}", self.duration.as_secs()));
         string.push_str(&format!(" {:0>3}", (self.elevation * 10.0) as u16));
         string.push_str(&format!(" {:0>3}", (self.azimuth * 10.0) as u16));
-        //string.push_str(&format!("     {}", (self.refsv * 10.0) as u16));
+        string.push_str(&format!(" {}", (self.refsv  * 1.0E10) as i32));
+        string.push_str(&format!(" {}", (self.srsv   * 1.0E13) as i32));
+        string.push_str(&format!(" {}", (self.refsys * 1.0E10) as i32));
+        string.push_str(&format!(" {}", (self.srsys  * 1.0E13) as i32));
+        string.push_str(&format!(" {}", (self.dsg    * 1.0E10) as i32));
+        string.push_str(&format!(" {}", self.ioe));
+        string.push_str(&format!(" {}", (self.mdtr   * 1.0E10) as i32));
+        string.push_str(&format!(" {}", (self.smdt   * 1.0E13) as i32));
+        string.push_str(&format!(" {}", (self.mdio   * 1.0E10) as i32));
+        string.push_str(&format!(" {}", (self.smdi   * 1.0E13) as i32));
+        
+        if let Some((msio, smsi, isg)) = self.get_ionospheric_parameters() {
+            string.push_str(&format!(" {}", (msio * 1.0E10) as i32));
+            string.push_str(&format!(" {}", (smsi * 1.0E13) as i32));
+            string.push_str(&format!(" {}", (isg *  1.0E10) as i32))       
+        }
+        string.push_str(&format!(" {:2X}", self.fr));
+        string.push_str(&format!(" {:2X}", self.hc));
+        string.push_str(&format!(" {}", self.frc));
+        if let Ok(crc) = calc_crc(&string) {
+            string.push_str(&format!(" {:2X}", crc))
+        } 
         fmt.write_str(&string)
     }
 }
@@ -535,19 +560,4 @@ mod test {
             )
         )
     }
-
-    #[test]
-    /// Tests CRC calculation method
-    fn test_crc_calc() {
-        let content = vec![
-            "R24 FF 57000 000600  780 347 394 +1186342 +0 163 +0 40 2 141 +22 23 -1 23 -1 29 +2 0 L3P"
-        ];
-        let expected = vec![0x0F];
-        for i in 0..content.len() {
-            let ck = calc_crc(content[i])
-                .unwrap();
-            let expect = expected[i];
-            assert_eq!(ck,expect,"Failed for \"{}\", expect \"{}\" but \"{}\" locally computed",content[i],expect,ck)
-        }
-    }
 }
